from ngsolve import *
from netgen.csg import *
import netgen.meshing as meshing


geo = CSGeometry()

spcurve = SplineCurve2d()
pts = [ (0,0), (0,1), (1,1), (2.7,0.99), (3, 0.1), (3,0) ]
for p in pts:
    spcurve.AddPoint(*p)

spcurve.AddSegment(0,1,2)
spcurve.AddSegment(2,3,4)
spcurve.AddSegment(4,5)

rev = Revolution(Pnt(0,0,-1),Pnt(0,0,1),spcurve).bc("rev") 
geo.Add (rev)

mesh = geo.GenerateMesh(maxh=0.6, perfstepsend=meshing.MeshingStep.MESHSURFACE)
mesh = Mesh(mesh)
mesh.Curve(3)
Draw(mesh)

fes = H1(mesh, order=3)
u,v = fes.TnT()
a = BilinearForm(fes)
a += grad(u).Trace()*grad(v).Trace()*ds + u*v*ds
a.Assemble()

m = BilinearForm(fes)
m += u*v*ds
m.Assemble()

ainv = a.mat.Inverse(inverse="sparsecholesky")
num = 35
evals, evecs = solvers.PINVIT(a.mat, m.mat, pre=ainv, num=num, maxit=10)

gfu = GridFunction(fes, multidim=num)
for i in range(num):
    gfu.vecs[i].data = evecs[i]
Draw (gfu)



